"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const express_1 = __importDefault(require("express"));
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const context_1 = require("../middleware/context");
const cors_1 = require("../middleware/cors");
const errors_1 = require("../middleware/errors");
const Log_1 = require("../utils/Log");
const upgrade_1 = require("../utils/upgrade");
async function Paperhat(config) {
    const log = new Log_1.Log();
    const upgradeOk = await upgrade_1.upgrade(config.upgrade, log.getSub('upgrade'));
    if (!upgradeOk) {
        process.exit(1);
    }
    log.info('Paperhat server starting...');
    const { port, appsRoot, jsonSizeLimit } = config.server;
    const api = express_1.default();
    api.set('x-powered-by', false);
    api.use(cookie_parser_1.default());
    api.use(context_1.useContext(config));
    api.use(cors_1.useCors(config.server));
    api.use(express_1.default.json({ limit: jsonSizeLimit }));
    if (config.init) {
        await config.init(api);
    }
    startApps(api, appsRoot, log);
    api.use(errors_1.handleErrors);
    log.info(`Paperhat server now listening at ${port}`);
    api.listen(port);
}
exports.Paperhat = Paperhat;
function startApps(api, appsRoot, log) {
    const dirs = fs_1.default.readdirSync(appsRoot, {
        encoding: 'utf8',
        withFileTypes: true,
    });
    for (const dir of dirs) {
        if (!dir.isDirectory())
            continue;
        const appId = dir.name;
        const appFile = path_1.default.join(appsRoot, appId, `${appId}.js`);
        if (!fs_1.default.existsSync(appFile))
            continue;
        log.info('Hooking app', appId);
        const hookApp = require(appFile).default;
        const app = express_1.default.Router();
        hookApp(app);
        api.use(`/${appId}`, app);
    }
}
