"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function mapEndpoints(app, mappings) {
    for (const endpoint in mappings) {
        const path = '/' + endpoint;
        const mapping = mappings[endpoint];
        for (const method in mapping) {
            const functions = mapping[method];
            const last = functions.length - 1;
            const handler = functions[last];
            const middle = functions.slice(0, last);
            app[method].call(app, path, [...middle, wrapHandler(handler)]);
        }
    }
}
exports.mapEndpoints = mapEndpoints;
const SendResult = {
    number: (res, status) => res.sendStatus(status),
    object: (res, data) => res.json({ data }),
    string: (res, fail) => res.json({ fail }),
    boolean: (res, success) => res.json({ success }),
    undefined: res => res.sendStatus(200),
};
function wrapHandler(handler) {
    return async (req, res, next) => {
        try {
            const result = await handler(req, res, next);
            if (!res.writableEnded) {
                const send = SendResult[typeof result];
                send(res, result);
            }
        }
        catch (e) {
            next(e);
        }
    };
}
