"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const jwt = __importStar(require("jsonwebtoken"));
class User {
    constructor(isProd, config, req, res) {
        this.isProd = isProd;
        this.config = config;
        this.req = req;
        this.res = res;
    }
    authenticate() {
        const { cookieName } = this.config;
        const { [cookieName]: token } = this.req.cookies;
        const payload = this.verify(token);
        if (!payload) {
            this.logout();
            return;
        }
        const { user, hard } = payload;
        this.data = user;
        this.state = Date.now() <= hard ? 'hard' : 'soft';
        this.setCookie(token);
    }
    login(data, hard = false) {
        this.data = data;
        this.state = hard ? 'hard' : 'soft';
        const { hardAge, jwtSecret } = this.config;
        const payload = {
            hard: hard ? Date.now() + hardAge : 0,
            user: {},
        };
        for (const field of this.config.payloadFields) {
            payload.user[field] = data[field];
        }
        const token = jwt.sign(payload, jwtSecret);
        this.setCookie(token);
    }
    logout() {
        this.state = 'guest';
        this.data = null;
        this.res.clearCookie(this.config.cookieName);
    }
    isGuest() {
        return this.state === 'guest';
    }
    isHard() {
        return this.state === 'hard';
    }
    get() {
        return this.data;
    }
    getRemoteIp() {
        return this.req.get('X-Client-IP');
    }
    // INTERNAL
    verify(token) {
        if (!token)
            return null;
        try {
            const { jwtSecret } = this.config;
            const payload = jwt.verify(token, jwtSecret);
            return payload;
        }
        catch (e) {
            return null;
        }
    }
    setCookie(token) {
        this.res.cookie(this.config.cookieName, token, {
            ...this.config.cookie,
            httpOnly: this.isProd,
            secure: this.isProd,
            sameSite: 'lax',
        });
    }
}
exports.User = User;
