import fs from 'fs';
import path from 'path';

import express, { Application } from 'express';
import cookieParser from 'cookie-parser';

import { useContext } from '../middleware/context';
import { useCors } from '../middleware/cors';
import { handleErrors } from '../middleware/errors';
import { Log } from '../utils/Log';
import { upgrade } from '../utils/upgrade';
import { Config } from './Config';

export async function Paperhat(config: Config) {
  const log = new Log();

  const upgradeOk = await upgrade(config.upgrade, log.getSub('upgrade'));
  if (!upgradeOk) {
    process.exit(1);
  }

  log.info('Paperhat server starting...');

  const { port, appsRoot, jsonSizeLimit } = config.server;
  const api = express();
  api.set('x-powered-by', false);

  api.use(cookieParser());
  api.use(useContext(config));
  api.use(useCors(config.server));
  api.use(express.json({ limit: jsonSizeLimit }));
  if (config.init) {
    await config.init(api);
  }

  startApps(api, appsRoot, log);
  api.use(handleErrors);

  log.info(`Paperhat server now listening at ${port}`);
  api.listen(port);
}

function startApps(api: Application, appsRoot: string, log: Log) {
  const dirs = fs.readdirSync(appsRoot, {
    encoding: 'utf8',
    withFileTypes: true,
  });

  for (const dir of dirs) {
    if (!dir.isDirectory()) continue;

    const appId = dir.name;
    const appFile = path.join(appsRoot, appId, `${appId}.js`);
    if (!fs.existsSync(appFile)) continue;

    log.info('Hooking app', appId);
    const hookApp = require(appFile).default;
    const app = express.Router();
    hookApp(app);

    api.use(`/${appId}`, app);
  }
}
