import { Application, Handler, Response } from 'express';

interface Mappings {
  [endpoint: string]: {
    options?: Handler[];
    head?: Handler[];
    get?: Handler[];
    post?: Handler[];
    put?: Handler[];
    patch?: Handler[];
    delete?: Handler[];
  };
}

export function mapEndpoints(app: Application, mappings: Mappings) {
  for (const endpoint in mappings) {
    const path = '/' + endpoint;
    const mapping = mappings[endpoint];
    for (const method in mapping) {
      const functions = mapping[method];
      const last = functions.length - 1;
      const handler = functions[last];
      const middle = functions.slice(0, last);
      app[method].call(app, path, [...middle, wrapHandler(handler)]);
    }
  }
}

const SendResult: Record<string, (res: Response, result: any) => void> = {
  number: (res, status) => res.sendStatus(status),
  object: (res, data) => res.json({ data }),
  string: (res, fail) => res.json({ fail }),
  boolean: (res, success) => res.json({ success }),
  undefined: res => res.sendStatus(200),
};

function wrapHandler(handler: Handler): Handler {
  return async (req, res, next) => {
    try {
      const result = await handler(req, res, next);
      if (!res.writableEnded) {
        const send = SendResult[typeof result];
        send(res, result);
      }
    } catch (e) {
      next(e);
    }
  };
}
