export interface Shape {
  [key: string]: Primitive | Complex;
}

export type Primitive = 'string' | 'number' | 'boolean' | 'any' | 'forbidden';

export type ComplexBase = {
  optional?: boolean;
  check?: Validator;
};

export type Complex = ComplexBase &
  (
    | { type: 'boolean' }
    | ({ type: 'string' } & StringConstraints)
    | ({ type: 'number' } & NumberConstraints)
    | ({ type: 'object' } & ObjectConstraints)
    | ({ type: 'array' } & ArrayConstraints)
    | ({ type: 'enum' } & EnumConstraints)
  );

export type Validator = (value: any, obj: any) => boolean;

export type StringConstraints = {
  minLength?: number;
  maxLength?: number;
  pattern?: RegExp;
};

export type NumberConstraints = {
  min?: number;
  max?: number;
  integer?: boolean;
};

export type ObjectConstraints = {
  shape: Shape;
};

export type ArrayConstraints = {
  of: Primitive | Complex;
  minLength?: number;
  maxLength?: number;
};

export type EnumConstraints = {
  of: string[];
};
