import { Handler } from 'express';
import { Shape } from './validate-json/types';
import { Validator } from './validate-json/Validator';

export { Shape } from './validate-json/types';
export { Validator } from './validate-json/Validator';

const jsonMemo = new WeakMap<Shape, Handler>();
const queryMemo = new WeakMap<Shape, Handler>();

const verboseDev = (arg?: boolean) =>
  typeof arg === 'boolean' ? arg : process.env.NODE_ENV !== 'prod';

export function validateJson(shape: Shape, verbose?: boolean): Handler {
  if (!jsonMemo.has(shape)) {
    const validator = new Validator(shape, verboseDev(verbose));
    const handler: Handler = (req, res, next) => {
      if (!validator.validate(req.body)) {
        next(400);
      } else {
        next();
      }
    };

    jsonMemo.set(shape, handler);
  }

  return jsonMemo.get(shape);
}

export function validateQuery(shape: Shape, verbose?: boolean): Handler {
  if (!queryMemo.has(shape)) {
    const validator = new Validator(shape, verboseDev(verbose));
    const handler: Handler = (req, res, next) => {
      if (!validator.validate(req.query)) {
        next(400);
      } else {
        next();
      }
    };

    queryMemo.set(shape, handler);
  }

  return queryMemo.get(shape);
}
