import { NextFunction, Request, Response, Handler } from 'express';

import { Config } from '../exports/Config';
import { SqlClient } from '../exports/SqlClient';
import { Log } from '../utils/Log';
import { Sql } from './Sql';
import { User } from './User';

const isProd = process.env.NODE_ENV === 'prod';

export interface Context {
  log: Log;
  sql: SqlClient;
  user: User;
  data: any;
}

export function useContext(config: Config): Handler {
  Sql.createPool(config.sql);

  return (req: Request, res: Response, next: NextFunction) => {
    req.context = {
      data: {},
      log: new Log(),
      sql: new Sql(),
      user: new User(isProd, config.user, req, res),
    };
    req.context.user.authenticate();
    next();
  };
}
