export function applyMacros(sql: string): string {
  return sql.replace(MACRO, (_, macro, args) => {
    const params: MacroParams = {};
    for (const pair of args.trim().split(' ')) {
      const [name, value] = pair.split(':');
      params[name] = value || true;
    }
    return Macros[macro].call(undefined, params);
  });
}

const MACRO = /\/\* !([A-Z]+) (.*)\*\//g;

const ID_TYPE = 'VARCHAR(24)';

type MacroParams = Record<string, string | boolean>;
type SqlMacro = (params: MacroParams) => string;

const Macros: Record<string, SqlMacro> = {
  ENV: ({ name }) => process.env[name as string],

  ID: () => `id ${ID_TYPE} PRIMARY KEY`,

  FKEY: ({ table, soft, pk, type = ID_TYPE, col = 'id' }) =>
    [
      type,
      soft ? 'DEFAULT NULL' : 'NOT NULL',
      pk ? 'PRIMARY KEY' : '',
      'REFERENCES',
      `${table}(${col})`,
      soft ? 'ON DELETE SET NULL' : 'ON DELETE CASCADE',
    ].join(' '),

  USER: ({ soft, pk }) =>
    Macros.FKEY({ table: 'paperhat.users', type: 'INT', soft, pk }),
};
