import { QueryConfig, QueryResult, PoolConfig } from 'pg';
import { SqlClient, SqlCursor } from '../exports/SqlClient';
interface Connection {
    query<T>(query: string, values?: any[]): Promise<T>;
    query<T>(config: QueryConfig): Promise<QueryResult<T>>;
    release?(): void;
}
export interface SqlConfig {
    pool: PoolConfig;
}
export declare class Sql implements SqlClient {
    private db;
    private static pool;
    static createPool(config: SqlConfig): void;
    constructor(db?: Connection);
    one<T = any>(strings: TemplateStringsArray, ...merges: any[]): Promise<T>;
    q<T = any>(strings: TemplateStringsArray, ...merges: any[]): Promise<T[]>;
    cur<T = any>(strings: TemplateStringsArray, ...merges: any[]): Promise<SqlCursor<T>>;
    raw<T = any>(query: string, values?: any[]): Promise<T[]>;
    get<T = any>(table: string, value: any, matchBy?: string): Promise<T>;
    value<T = any>(sql: string, values: any[], name?: string): Promise<T>;
    insert<T = any>(table: string, row: Partial<T>, returning?: string): Promise<T>;
    update<T = any>(table: string, row: Partial<T>, matchBy?: string): Promise<boolean>;
    delete<T = any>(table: string, row: Partial<T>, matchBy?: string): Promise<boolean>;
    transaction<R = any>(block: (tx: Sql) => Promise<R | boolean>): Promise<R>;
    arrayTransaction<T = any, R = any>(items: T[], operation: (tx: Sql, item: T, i: number) => Promise<R>): Promise<R[]>;
    private tagQuery;
    private exec;
}
export declare function buildQuery(strings: TemplateStringsArray, merges: any[]): QueryConfig;
export {};
